@@ -29,12 +29,10 @@
 #include "qgscoordinatetransform.h"
 #include "qgsellipsoidutils.h"
 #include "qgsunittypes.h"
-
-Q_GUI_EXPORT extern int qt_defaultDpiX();
-
+#include "qgspainting.h"
 
 QgsMapSettings::QgsMapSettings()
-  : mDpi( qt_defaultDpiX() ) // DPI that will be used by default for QImage instances
+  : mDpi( QgsPainting::qtDefaultDpiX() ) // DPI that will be used by default for QImage instances
   , mSize( QSize( 0, 0 ) )
   , mBackgroundColor( Qt::white )
   , mSelectionColor( Qt::yellow )
@@ -236,6 +234,18 @@ void QgsMapSettings::updateDerived()
 
 }
 
+void QgsMapSettings::matchRasterizedRenderingPolicyToFlags()
+{
+  if ( !mFlags.testFlag( Qgis::MapSettingsFlag::ForceVectorOutput )
+       && mFlags.testFlag( Qgis::MapSettingsFlag::UseAdvancedEffects ) )
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::Default;
+  else if ( mFlags.testFlag( Qgis::MapSettingsFlag::ForceVectorOutput )
+            && mFlags.testFlag( Qgis::MapSettingsFlag::UseAdvancedEffects ) )
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::PreferVector;
+  else if ( mFlags.testFlag( Qgis::MapSettingsFlag::ForceVectorOutput )
+            && !mFlags.testFlag( Qgis::MapSettingsFlag::UseAdvancedEffects ) )
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::ForceVector;
+}
 
 QSize QgsMapSettings::outputSize() const
 {
@@ -325,6 +335,23 @@ QList<QgsMapLayer *> QgsMapSettings::layers( bool expandGroupLayers ) const
   return result;
 }
 
+template<typename T>
+QVector<T> QgsMapSettings::layers() const
+{
+  const QList<QgsMapLayer *> actualLayers = _qgis_listQPointerToRaw( mLayers );
+
+  QVector<T> layers;
+  for ( QgsMapLayer *layer : actualLayers )
+  {
+    T tLayer = qobject_cast<T>( layer );
+    if ( tLayer )
+    {
+      layers << tLayer;
+    }
+  }
+  return layers;
+}
+
 void QgsMapSettings::setLayers( const QList<QgsMapLayer *> &layers )
 {
   // filter list, removing null layers and non-spatial layers
@@ -378,6 +405,7 @@ bool QgsMapSettings::setEllipsoid( const QString &ellipsoid )
 void QgsMapSettings::setFlags( Qgis::MapSettingsFlags flags )
 {
   mFlags = flags;
+  matchRasterizedRenderingPolicyToFlags();
 }
 
 void QgsMapSettings::setFlag( Qgis::MapSettingsFlag flag, bool on )
@@ -386,6 +414,7 @@ void QgsMapSettings::setFlag( Qgis::MapSettingsFlag flag, bool on )
     mFlags |= flag;
   else
     mFlags &= ~( static_cast< int >( flag ) );
+  matchRasterizedRenderingPolicyToFlags();
 }
 
 Qgis::MapSettingsFlags QgsMapSettings::flags() const
@@ -403,6 +432,16 @@ Qgis::DistanceUnit QgsMapSettings::mapUnits() const
   return mScaleCalculator.mapUnits();
 }
 
+Qgis::ScaleCalculationMethod QgsMapSettings::scaleMethod() const
+{
+  return mScaleCalculator.method();
+}
+
+void QgsMapSettings::setScaleMethod( Qgis::ScaleCalculationMethod method )
+{
+  mScaleCalculator.setMethod( method );
+  updateDerived();
+}
 
 bool QgsMapSettings::hasValidSettings() const
 {
@@ -900,3 +939,29 @@ void QgsMapSettings::setElevationShadingRenderer( const QgsElevationShadingRende
 {
   mShadingRenderer = elevationShadingRenderer;
 }
+
+Qgis::RasterizedRenderingPolicy QgsMapSettings::rasterizedRenderingPolicy() const
+{
+  return mRasterizedRenderingPolicy;
+}
+
+void QgsMapSettings::setRasterizedRenderingPolicy( Qgis::RasterizedRenderingPolicy policy )
+{
+  mRasterizedRenderingPolicy = policy;
+  switch ( mRasterizedRenderingPolicy )
+  {
+    case Qgis::RasterizedRenderingPolicy::Default:
+      mFlags.setFlag( Qgis::MapSettingsFlag::ForceVectorOutput, false );
+      mFlags.setFlag( Qgis::MapSettingsFlag::UseAdvancedEffects, true );
+      break;
+    case Qgis::RasterizedRenderingPolicy::PreferVector:
+      mFlags.setFlag( Qgis::MapSettingsFlag::ForceVectorOutput, true );
+      mFlags.setFlag( Qgis::MapSettingsFlag::UseAdvancedEffects, true );
+      break;
+    case Qgis::RasterizedRenderingPolicy::ForceVector:
+      mFlags.setFlag( Qgis::MapSettingsFlag::ForceVectorOutput, true );
+      mFlags.setFlag( Qgis::MapSettingsFlag::UseAdvancedEffects, false );
+      break;
+  }
+}
+
